require 'uri'

class LearningObjectsController < ApplicationController
  include Reportable

  before_action :authenticate_user!, except: [:index, :show]
  before_action :set_learning_object, only: [:show, :edit, :update,
                                             :destroy, :like, :bookmarks,
                                             :collections, :upload, :upload_link, :download]
  before_action :set_learning_object_type, only: :show
  after_action :increment_learning_object_views, only: [:show]

  # GET /learning_objects/1
  # GET /learning_objects/1.json
  def show
    @liked = !@learning_object.likes.where(user: current_user).blank? if user_signed_in?
  end

  # GET /learning_objects/new
  def new
    @learning_object = LearningObject.new
    @school_levels = ['Educação Infantil', 'Ensino Fundamental', 'Ensino Médio']
    @topics = Topic.defaults
    @types = LearningObject.object_types
    @languages = Language.all
  end

  # GET /learning_objects/1/edit
  def edit
  end

  # POST /learning_objects
  # POST /learning_objects.json
  def create
    @draft = ::LearningObject::DraftBuilder.build current_user, learning_object_params
    publisher = LearningObjectPublisher.new(DspaceService.create_client)

    respond_to do |format|
      if publisher.create_draft @draft
        # go to file submission page
        format.html { success_redirect @draft }
      else
        format.html { redirect_to new_learning_object_path, notice: 'Não foi possível criar o objeto.' }
      end
    end
  end

  def upload
  end

  # PATCH/PUT /learning_objects/1
  # PATCH/PUT /learning_objects/1.json
  def update
    respond_to do |format|
      if LearningObject.update(learning_object_params)
        format.html { redirect_to @learning_object, notice: 'Learning object was successfully updated.' }
      else
        format.html { render :edit }
      end
    end
  end

  # DELETE /learning_objects/1
  # DELETE /learning_objects/1.json
  def destroy
    LearningObject.destroy @learning_object

    respond_to do |format|
      format.html { redirect_to learning_objects_url, notice: 'Learning object was successfully destroyed.' }
    end
  end

  # POST /learning_objects/1/like
  def like
    if @learning_object.liked? current_user
      @learning_object.dislike current_user
    else
      @learning_object.like current_user
    end

    if request.xhr?
      render json: {count: @learning_object.likes.count, id: params[:id]}
    end
  end

  def download
    @learning_object.download current_user
    redirect_to @learning_object.retrieve_link
  end

  # POST /learning_objects/1/bookmarks
  def bookmarks
    bookmarks = current_user.bookmarks
    bookmarks.add @learning_object
    Collection.save_learning_objects bookmarks

    if request.xhr?
      render json: {id: params[:id]}
    end
  end

  # GET /learning_objects/1/collections.json
  def collections
    @collections = @learning_object.collections
  end

  private

  # Use callbacks to share common setup or constraints between actions.
  def set_learning_object
    @learning_object = LearningObject.find params[:id]
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def learning_object_params
    params[:learning_object].permit(:author, :name, :type, :description, :school_level, :language, topics: [])
  end

  def increment_learning_object_views
    View.create(viewable: @learning_object, user: current_user) if user_signed_in?
  end

  def success_redirect(learning_object)
    id = learning_object.id
    if learning_object.type == "Link Externo"
      redirect_to learning_object_path(id: id) , notice: 'Seu objeto foi criado e publicado com sucesso!'
    else
      redirect_to upload_learning_object_path(id: id) , notice: 'Seu objeto foi criado! Para que ele seja publicado é necessário enviar o arquivo.'
    end
  end
end
